local NUM_TRADING_SLOTS = 7
local interfaceItem = {}
local interfaceIndex = 0

local addonName, player = ...

player.signature = "|c8788EEff" .. addonName .. ":|r "

local TRADEFILL = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0")

local OptionFrame = CreateFrame("Frame", "MyIconButtonFrame", TradeFrame)
local ButtonA = CreateFrame("Button", "MyTradeButton", OptionFrame)
local ButtonB = CreateFrame("Button", "MyTradeButton", OptionFrame)
local Clear = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")


local options = {
    name = addonName,
    handler = TRADEFILL,
    type = "group",
    childGroups = "tab",
    args = {
        menu = {
            type = "group",
            name = "",
            order = 0,
            args = {
                toggleBoxB = {
                    type = 'toggle',
                    name = "Show only full stacks",
                    desc = "Hide stacks that are not full in dropdown menu",
                    get = function(info) return TRADEFILL.db.profile.setting.show end,
                    set = function(info, value) TRADEFILL.db.profile.setting.show = value end,
                    order = 0,
                },
                toggleBoxA = {
                    type = 'toggle',
                    name = "|cFFFF0000Standby|r",
                    desc = "Don't AutoFill tradewindow\n\nCommands:\n/tradefillon\n/tradefilloff",
                    get = function(info) return TRADEFILL.db.profile.setting.standby end,
                    set = function(info, value) TRADEFILL.db.profile.setting.standby = value end,
                    order = 1,
                },                
                separatorA = {
                    type = "description",
                    name = " ",
                    order = 2,
                },
                iconA = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TRADEFILL:GetName("A") end,
                    image = function(info) return TRADEFILL:SetIconImage("A") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 3,
                },
                itemA = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return TRADEFILL:SetOptionItem("A") end,
                    get = function() return TRADEFILL:GetInputItem("A") end,
                    set = function(info, value) TRADEFILL:SetSelectItem("A", tonumber(value)) end,
                    order = 4,
                },
                separatorB = {
                    type = "description",
                    name = " ",
                    order = 5,
                },
                iconB = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TRADEFILL:GetName("B") end,
                    image = function(info) return TRADEFILL:SetIconImage("B") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 6,
                },
                itemB = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return TRADEFILL:SetOptionItem("B") end,
                    get = function() return TRADEFILL:GetInputItem("B") end,
                    set = function(info, value) TRADEFILL:SetSelectItem("B", tonumber(value)) end,
                    order = 7,
                },
                separatorC = {
                    type = "description",
                    name = " ",
                    order = 8,
                },
                toggleBoxC = {
                    type = 'toggle',
                    name = "Only full stacks",
                    desc = "Fill the trade window only with full stacks",
                    get = function(info) return TRADEFILL.db.profile.setting.full end,
                    set = function(info, value) TRADEFILL.db.profile.setting.full = value end,
                    order = 9,
                },
                toggleBoxD = {
                    type = 'toggle',
                    name = "Disable buttons",
                    desc = "Hide extra buttons at trade window",
                    get = function(info) return TRADEFILL.db.profile.setting.button end,
                    set = function(info, value) TRADEFILL.db.profile.setting.button = value end,
                    order = 10,
                },
                toggleBoxE = {
                    type = 'toggle',
                    name = "Disable message",
                    desc = "Hide chat message that show info about the trading conditions",
                    get = function(info) return TRADEFILL.db.profile.setting.message end,
                    set = function(info, value) TRADEFILL.db.profile.setting.message = value end,
                    order = 11,
                },
            },
            inline = true,
        },
        stacks = {
            type = "group",
            name = "Stacks",
            desc = "The number of stacks you want to trade for each class",
            order = 1,
            args = {
            },
        },
        group = {
            type = "group",
            name = "Condition",
            desc = "When to fill the trade window",
            order = 2,
            args = {
                group2 = {
                    type = "group",
                    name = "First condition",
                    desc = "Player has to meet those conditions",
                    order = 0,
                    args = {
                        toggleBoxA = {
                            type = 'toggle',
                            name = "Strangers",
                            desc = "AutoFill trade window with anyone when your not in a group",
                            get = function() return TRADEFILL.db.profile.setting.solo end,
                            set = function(info, value) TRADEFILL.db.profile.setting.solo = value end,
                            order = 0,
                        },
                        toggleBoxB = {
                            type = 'toggle',
                            name = "Party members",
                            desc = "AutoFill trade window with party members",
                            get = function() return TRADEFILL.db.profile.setting.party end,
                            set = function(info, value) TRADEFILL.db.profile.setting.party = value end,
                            order = 1,
                        },
                        toggleBoxC = {
                            type = 'toggle',
                            name = "Raid members",
                            desc = "AutoFill trade window with raid members",
                            get = function() return TRADEFILL.db.profile.setting.raid end,
                            set = function(info, value) TRADEFILL.db.profile.setting.raid = value end,
                            order = 2,
                        },
                    },
                },
                group = {
                    type = "group",
                    name = "Second condition",
                    desc = "Player also has to meet those conditions",
                    order = 1,
                    args = {
                        toggleBoxD = {
                            type = 'toggle',
                            name = "Guild members",
                            desc = "Only with guild members",
                            get = function() return TRADEFILL.db.profile.setting.guild end,
                            set = function(info, value) TRADEFILL.db.profile.setting.guild = value end,
                            order = 0,
                        },
                        toggleBoxE = {
                            type = 'toggle',
                            name = "Required level",
                            desc = "Only when the player has the required item level",
                            get = function() return TRADEFILL.db.profile.setting.require end,
                            set = function(info, value) TRADEFILL.db.profile.setting.require = value end,
                            order = 1,
                        },
                        toggleBoxF = {
                            type = 'toggle',
                            name = "Level " .. GetMaxPlayerLevel(),
                            desc = "Only when the player is level cap",
                            get = function() return TRADEFILL.db.profile.setting.level end,
                            set = function(info, value) TRADEFILL.db.profile.setting.level = value end,
                            order = 2,
                        },
                        toggleBoxG = {
                            type = 'toggle',
                            name = "|cFFFF0000Master looter|r",
                            desc = "Except when you the loot master",
                            get = function() return TRADEFILL.db.profile.setting.master end,
                            set = function(info, value) TRADEFILL.db.profile.setting.master = value end,
                            order = 3,
                        },
                    },
                },
            },
        },
        exclude = {
            type = "group",
            name = "Exclude",
            desc = "When not to fill the trade window",
            order = 3,
            args = {
                player = {
                    type = "group",
                    name = "Players",
                    desc = "Player names",
                    order = 0,
                    args = {
                        inputA = {
                            type = 'input',
                            name = "Player names",
                            desc = "Don't fill trade window with those players",
                            get = function() return TRADEFILL.db.profile.setting.players end,
                            set = function(info, value) TRADEFILL.db.profile.setting.players = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n]*$",
                            usage = "Player names can only contains letters",
                            multiline = 10,
                            order = 0,
                        },
                    },
                },
                guild = {
                    type = "group",
                    name = "Guilds",
                    desc = "Guild names",
                    order = 1,
                    args = {
                        inputA = {
                            type = 'input',
                            name = "Guild names",
                            desc = "Don't fill trade window with players from these guilds",
                            get = function() return TRADEFILL.db.profile.setting.guilds end,
                            set = function(info, value) TRADEFILL.db.profile.setting.guilds = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n ]*$",
                            usage = "Guild names can only contains letters",
                            multiline = 10,
                            order = 1,
                        },
                    },
                },
            },
        },
    },
}

function TRADEFILL:OnInitialize()
    for index, class in pairs(classes) do
        options.args.stacks.args[class] = {
            type = "group",
            name = setClassName(class),
            order = index,
            args = {
                iconA = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TRADEFILL:GetName("A") end,
                    image = function(info) return TRADEFILL:SetIconImage("A") end,
                    imageWidth = 16,
                    imageHeight = 16,
                    order = 0,
                },
                selectedOptionA = {
                    type = "select",
                    width = 0.75,
                    name = "Number of stacks",
                    desc = "Choose an amount of stack you want to trade",
                    values = function() return TRADEFILL:SetOptionStack("A") end,
                    get = function() return tostring(TRADEFILL.db.profile.stacks[class].a) end,
                    set = function(info, value) TRADEFILL.db.profile.stacks[class].a = tonumber(value) TRADEFILL:UpdateOptionStack(class, "A", tonumber(value)) end,
                    order = 1,
                },
                separatorA = {
                    type = "description",
                    name = " ",
                    order = 2,
                },
                iconB = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TRADEFILL:GetName("B") end,
                    image = function(info) return TRADEFILL:SetIconImage("B") end,
                    imageWidth = 16,
                    imageHeight = 16,
                    order = 3,
                },
                selectedOptionB = {
                    type = "select",
                    width = 0.75,
                    name = "Number of stacks",
                    desc = "Choose an amount of stack you want to trade",
                    values = function() return TRADEFILL:SetOptionStack("B") end,
                    get = function() return tostring(TRADEFILL.db.profile.stacks[class].b) end,
                    set = function(info, value) TRADEFILL.db.profile.stacks[class].b = tonumber(value) TRADEFILL:UpdateOptionStack(class, "B", tonumber(value)) end,
                    order = 4,
                },
            },
        }
    end

    LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options)
    local AceConfigDialog = LibStub("AceConfigDialog-3.0")
    AceConfigDialog:AddToBlizOptions(addonName, addonName)

    local defaultsItems = {
        profile = {
            item = {
                a = {
                    id = 0,
                    name = "",
                },
                b = {
                    id = 0,
                    name = "",
                },
            },
            stacks = {
                ["**"] = {
                    a = 0,
                    b = 0
                },
            },
            setting = {
                solo = false,
                party = false,
                raid = false,
                standby = false,
                show = true,
                full = true,
                button = false,
                message = false,
                require = false,
                level = false,
                master = false,
                guild = false,
                guilds = "",
                players = ""
            }
        }
    }

    self.db = LibStub("AceDB-3.0"):New("TradeFillDB", defaultsItems)

    player.data = self.db.profile

    local faction = UnitFactionGroup("player")
    local patch, _, _, _ = GetBuildInfo()
    local versionWithoutDot = patch:gsub("%.", "")

    if tonumber(string.sub(versionWithoutDot, 1, 3)) < 251 then
        if faction == "Alliance" then
            options.args.stacks.args["SHAMAN"].hidden = true
        elseif faction == "Horde" then
            options.args.stacks.args["PALADIN"].hidden = true
        end
    end

    OptionFrame:SetWidth(200)
    OptionFrame:SetHeight(600)
    OptionFrame:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0)


    local itemA = TRADEFILL.db.profile.item.a.id

    ButtonA:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 18, 62)
    ButtonA:SetSize(32, 32)
    ButtonA:SetScript("OnClick", function() fillTradeWindow(getItem(TRADEFILL.db.profile.item.a.id), tradePosition()) end)
    ButtonTextA = ButtonA:CreateFontString(nil, "OVERLAY", "NumberFontNormal")
    ButtonTextA:SetPoint("BOTTOMRIGHT", ButtonA, "BOTTOMRIGHT", -2, 3)

    if itemA == 0 then
        ButtonA:SetNormalTexture(GetItemIcon(134400))
    else
        ButtonA:SetNormalTexture(GetItemIcon(itemA))
    end

    TRADEFILL:enableDisableButton(itemA, "A")


    local itemB = TRADEFILL.db.profile.item.b.id

    ButtonB:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 18, 28)
    ButtonB:SetSize(32, 32)
    ButtonB:SetScript("OnClick", function() fillTradeWindow(getItem(TRADEFILL.db.profile.item.b.id), tradePosition()) end)
    ButtonTextB = ButtonB:CreateFontString(nil, "OVERLAY", "NumberFontNormal")
    ButtonTextB:SetPoint("BOTTOMRIGHT", ButtonB, "BOTTOMRIGHT", -2, 3)

    if itemB == 0 then
        ButtonB:SetNormalTexture(GetItemIcon(134400))
    else
        ButtonB:SetNormalTexture(GetItemIcon(itemB))
    end

    TRADEFILL:enableDisableButton(itemB, "B")

    Clear:SetText("Clear")
    Clear:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 73, 5)
    Clear:SetSize(75, 22)
    Clear:SetScript("OnClick", function() clearTade() end)

    self:RegisterChatCommand("tradefill", "SlashCommand")
    self:RegisterChatCommand("tradefillon", "On")
    self:RegisterChatCommand("tradefilloff", "Off")

    DEFAULT_CHAT_FRAME:AddMessage(string.format(player.signature .. "version %s Loaded.", C_AddOns.GetAddOnMetadata(addonName, "Version")))

    if _G.ItemDB ~= nil then
        local name, _ = UnitName("player")
        local _, class, _ = UnitClass("player")

        DEFAULT_CHAT_FRAME:AddMessage(player.signature .. "Thanks for using TradeFill before " .. setName(name, class) .. ",|cFF00FF00 but due to structural change you need to set it up again|r")
    end

    self:RegisterEvent("TRADE_SHOW")
end

function TRADEFILL:enableDisableClassSetting(value, option)

    if option == "A" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionA.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionA.disabled = false
            end
        end
    end

    if option == "B" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionB.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionB.disabled = false
            end
        end
    end
end

function TRADEFILL:enableDisableButton(value, option)

    if option == "A" then

        if value == 0 or TRADEFILL.db.profile.setting.button == true then
            ButtonA:Hide()
        else
            ButtonA:Show()
        end
    end

    if option == "B" then

        if value == 0 or TRADEFILL.db.profile.setting.button == true then
            ButtonB:Hide()
        else
            ButtonB:Show()
        end
    end

    if TRADEFILL.db.profile.item.a.id == 0 and TRADEFILL.db.profile.item.b.id == 0 or TRADEFILL.db.profile.button == true then
        Clear:Hide()
    else
        Clear:Show()
    end
end

function TRADEFILL:SetOptionStack(option)

    local selectOption = {}

    if option == "A" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = TRADEFILL.db.profile.item.a.id

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                selectOption[tostring(i)] = i
            end
        end
    end

    if option == "B" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = TRADEFILL.db.profile.item.b.id

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                selectOption[tostring(i)] = i
            end
        end
    end

    return selectOption
end

function TRADEFILL:SetOptionItem(option)

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                interfaceIndex = interfaceIndex + 1
                interfaceItem[interfaceIndex] = bagItem
            end
        end
    end

    local selectOption = {}

    selectOption["0"] = "None"

    for i, slot in pairs(interfaceItem) do

        local size = getItem(slot.itemID)

        if TRADEFILL.db.profile.setting.show == true then
            if slot.isBound == false and slot.stackCount == size.stack then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        else
            if slot.isBound == false then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        end

        if option == "A" and TRADEFILL.db.profile.item.a.id > 0 then
            local item = TRADEFILL.db.profile.item.a.id
            local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end

        if option == "B" and TRADEFILL.db.profile.item.b.id > 0 then
            local item = TRADEFILL.db.profile.item.b.id
            local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end
    end
    interfaceItem = {}
    interfaceIndex = 0
    return selectOption
end

function TRADEFILL:UpdateOptionStack(class, option, value)

    local stackA = self.db.profile.stacks[class].a
    local stackB = self.db.profile.stacks[class].b

    local newValue = NUM_TRADING_SLOTS - value

    if option == "A" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.db.profile.stacks[class].b = newValue
        end
    end

    if option == "B" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.db.profile.stacks[class].a = newValue
        end
    end
end

function TRADEFILL:GetInputItem(option)

    if option == "A" then
        local value = TRADEFILL.db.profile.item.a.id
        TRADEFILL:enableDisableClassSetting(value, option)
        TRADEFILL:enableDisableButton(value, option)
        return tostring(value)
    end

    if option == "B" then
        local value = TRADEFILL.db.profile.item.b.id
        TRADEFILL:enableDisableClassSetting(value, option)
        TRADEFILL:enableDisableButton(value, option)
        return tostring(value)
    end
end

function TRADEFILL:SetSelectItem(option, value)

    local item = getItem(value)

    if option == "A" then

        ButtonA:SetSize(32, 32)

        if value == 0 then
            ButtonA:SetNormalTexture(GetItemIcon(134400))
        else
            ButtonA:SetNormalTexture(GetItemIcon(value))
        end

        self.db.profile.item.a.name = item.name
        self.db.profile.item.a.id = value

        TRADEFILL:enableDisableClassSetting(value, option)
        TRADEFILL:enableDisableButton(value, option)
    end

    if option == "B" then

        ButtonB:SetSize(32, 32)

        if value == 0 then
            ButtonB:SetNormalTexture(GetItemIcon(134400))
        else
            ButtonB:SetNormalTexture(GetItemIcon(value))
        end

        self.db.profile.item.b.name = item.name
        self.db.profile.item.b.id = value

        TRADEFILL:enableDisableClassSetting(value, option)
        TRADEFILL:enableDisableButton(value, option)
    end
end

function TRADEFILL:GetName(option)

    local level = TRADEFILL.db.profile.setting.require

    if option == "A" then
        local item = getItem(self.db.profile.item.a.id)

        if item.name then
            if level == true then
                return item.link:gsub("[%[%]]", "") .. " |cFF008000 Requires Level " .. item.level .. "|r"
            else
                return item.link:gsub("[%[%]]", "")
            end
        else
            return "Unknown"
        end
    end

    if option == "B" then
        local item = getItem(self.db.profile.item.b.id)

        if item.name then
            if level == true then
                return item.link:gsub("[%[%]]", "") .. " |cFF008000 Requires Level " .. item.level .. "|r"
            else
                return item.link:gsub("[%[%]]", "")
            end
        else
            return "Unknown"
        end
    end
end

function TRADEFILL:SetIconImage(option)

    if option == "A" then
        local item = getItem(self.db.profile.item.a.id)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end

    if option == "B" then
        local item = getItem(self.db.profile.item.b.id)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end
end


function TRADEFILL:SlashCommand()
    InterfaceOptionsFrame_OpenToCategory(TRADEFILL)
end

function TRADEFILL:On()
    DEFAULT_CHAT_FRAME:AddMessage(player.signature .. "AutoFill is |cFF00FF00active|r")

    TRADEFILL.db.profile.setting.standby = false
end

function TRADEFILL:Off()
    DEFAULT_CHAT_FRAME:AddMessage(player.signature .. "AutoFill is |cFFFF0000standby|r")

    TRADEFILL.db.profile.setting.standby = true
end

ButtonA:SetScript("OnEnter", function(self)
    self:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
end)

ButtonB:SetScript("OnEnter", function(self)
    self:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
end)